@@ -1605,7 +1605,7 @@ void cmd_context::push() {
         throw ex;
     }
     catch (z3_exception & ex) {
-        throw cmd_exception(ex.msg());
+        throw cmd_exception(ex.what());
     }
 }
 
@@ -1768,7 +1768,7 @@ void cmd_context::check_sat(unsigned num_assumptions, expr * const * assumptions
             throw ex;
         }
         catch (z3_exception & ex) {
-            throw cmd_exception(ex.msg());
+            throw cmd_exception(ex.what());
         }
         get_opt()->set_status(r);
     }
@@ -1782,19 +1782,15 @@ void cmd_context::check_sat(unsigned num_assumptions, expr * const * assumptions
         try {
             r = m_solver->check_sat(num_assumptions, assumptions);
             if (r == l_undef && !m().inc()) {
-                m_solver->set_reason_unknown(eh);
+                m_solver->set_reason_unknown(eh, "canceled");
             }
         }
         catch (z3_error & ex) {
+            m_solver->set_reason_unknown(eh, ex);
             throw ex;
         }
         catch (z3_exception & ex) {
-            if (!m().inc()) {
-                m_solver->set_reason_unknown(eh);
-            }
-            else {
-                m_solver->set_reason_unknown(ex.msg());
-            }
+            m_solver->set_reason_unknown(eh, ex);
             r = l_undef;
         }
         m_solver->set_status(r);
@@ -1832,7 +1828,7 @@ void cmd_context::get_consequences(expr_ref_vector const& assumptions, expr_ref_
         throw ex;
     }
     catch (z3_exception & ex) {
-        m_solver->set_reason_unknown(ex.msg());
+        m_solver->set_reason_unknown(ex.what());
         r = l_undef;
     }
     m_solver->set_status(r);
