@@ -248,6 +248,11 @@ public:
     */
     virtual expr* congruence_next(expr* e) = 0;
 
+    /**
+       \brief try to solve for term e (when e is arithmetical).
+    */
+    virtual bool solve_for(expr* e, expr_ref& term) { return false; }
+
     /**
        \brief Display the content of this solver.
     */
