@@ -416,7 +416,7 @@ public:
             (*m_imp)(g, result);
         }
         catch (rewriter_exception & ex) {
-            throw tactic_exception(ex.msg());
+            throw tactic_exception(ex.what());
         }
     }
     
